/*******************************************************/
/*                                                     */
/* Computing Disinflation Scenaria based on            */
/* parameters of DSGE models                           */
/*                                                     */
/*******************************************************/

/* filename:    lpdisinf.g
** description: The program converts the DSGE parameters
**              into impulse response functions
** created:     
*/

new;
closeall;
library pgraph, user;
cls;

/* Global defaults 
*/
  __output = 0;

#include c:\projects\active\learnpol\gauss2\pathspec.g;
#include c:\projects\active\learnpol\gauss2\lpspec.g;


/******************************************************** 
** Import data
** series (nobs,3)
*/
#include c:\projects\active\learnpol\gauss2\loaddata.g

nobs  = rows(series_yT);  /* number of observations */
YY    = series_yT;   

/* Disinflation experiment
*/
Tend  = 81;
SS    = ones(12,1)~zeros(12,1);

/* Open files with Parameter Draws
*/
mhrun  = "2";

lpath  = parapath $+ "\\mhrun" $+ mhrun;
opath  = resupath $+ "\\mhrun" $+ mhrun;

ltheta  = lpath $+ "\\" $+ lmodel $+ lprior $+ dataselstr $+ subTstr $+ "pt";
lphi    = lpath $+ "\\" $+ lmodel $+ lprior $+ dataselstr $+ subTstr $+ "pp";

open fhtheta = ^ltheta for read;
open fhphi   = ^lphi for read;

/* Define some Constants
*/
nblock  = 100;  /* block size */
nuse    = 4;     /* use every nuse observation */

/* Initialize Output files
*/
oirf = opath $+ "\\" $+ lmodel $+ lprior $+ dataselstr $+ subTstr $+ "di";
create fhirf = ^oirf with IRF, 5*(rows(SS)+1), 8;
       
/* Initialization
*/ 
eofloop = 0;
loopct  = 1;

do until eofloop; 

   timestart = date;

   /* Read blocks of size nblock
   */
   thetasim = readr( fhtheta, nblock );
   phisim   = readr( fhphi, nblock);
   
   nread = rows(thetasim);

   indseq = seqa(1,1,nread);
   indseq = indseq % nuse;
   indseq = indseq .NE 0;
 
   thetasim = delif(thetasim,indseq);
   phisim   = delif(phisim,indseq);
   
   nread = rows(thetasim);

   irfsim = zeros(nread,5*(rows(SS)+1));
   
   j = 1;
   do until j > nread;
   
      /* For each element of the block, compute the desired statistic
      */
      
      theta = thetasim[j,.]';
      phi   = phisim[j,.]';
     
      yTirf = preddisinf(theta,phi,mspec,Tend,YY,SS);
      
      /* composition of yTirf:
      ** output growth, inflation, interest rate, expected inflation, regime prob
      */
           
      irfsim[j,.] = vec(yTirf)';
      
      j = j+1;
      
   endo;

   /* Write Output
   */
   wr = writer(fhirf,irfsim);

   locate 1,1;
   "Block" loopct;
   "Draws Processed:" nread;   

   timeend = date;
   "Elapsed Time"; etstr(ethsec(timestart,timeend));

   loopct = loopct + 1;
   eofloop = eof(fhtheta);

endo;

closeall fhtheta, fhphi, fhirf;
      
#include c:\projects\active\learnpol\gauss2\lpmod.src;
#include c:\projects\active\learnpol\gauss2\lpdisinf.src;
