/* filename:    lpgibbs1.g
** description: The program generates draws from the posterior
**              distribution via a Metropolis-within-Gibbs algorithm
**              
** created:     02/21/02
** modified:   
*/

new;
closeall;
library user, pgraph;
cls;

/* Global defaults 
*/
  __output = 1;
  _fcmptol = 1E-10;

#include c:\projects\active\learnpol\gauss2\pathspec.g
#include c:\projects\active\learnpol\gauss2\lpspec.g

/******************************************************** 
**  Configure the MCMC Algorithm
*/
mhrun   = "1";
cc0     = 2;
cc      = 0.15;
nblocks = 20;
nsim    = 100;
opath   = parapath $+ "\\mhrun" $+ mhrun;

/* bounds for theta
*/
_bounds =
    (1E-5 ~ 10      )|
    (1E-5 ~ 10      )|
    (1E-5 ~ 10      )|
    (1E-5 ~ 10      )|
    (1E-5 ~ 0.99999 )|
    (1E-5 ~ 10      )|
    (1E-5 ~ 10      )|
    (1E-5 ~ 0.99999 )|
    (1E-5 ~ 0.99999 )|
    (1E-5 ~ 0.99999 )|
    (1E-7 ~ 100     )|
    (1E-7 ~ 100     )|
    (1E-7 ~ 100     )|
    (1E-7 ~ 100     );

/******************************************************** 
** Import data
** series (nobs,3)
*/
#include c:\projects\active\learnpol\gauss2\loaddata.g

nobs  = rows(series_yT);  /* number of observations */
YY    = series_yT;   
SS    = series_sT;  

/******************************************************** 
** Import the information for the prior density
*/
priorfile = lmodel $+ lprior $+ dataselstr $+ subTstr $+ "prt.out";
load path = ^priopath prior[nparat,3] = ^priorfile;

pmean_t  = prior[.,1];
pstdd_t  = prior[.,2];
pshape_t = prior[.,3];
pmean_t  = pmean_t.*parat_maskinv[.,mspec] + parat_fix[.,mspec].*parat_mask[.,mspec];
pshape_t = pshape_t.*parat_maskinv[.,mspec];

priorfile = lmodel $+ lprior $+ dataselstr $+ subTstr $+ "prp.out";
load path = ^priopath prior[nparap,3] = ^priorfile;

pmean_p  = prior[.,1];
pstdd_p  = prior[.,2];

/******************************************************** 
** Load Posterior Mode Estimates
*/

lpara = postpath $+ "\\" $+ lmodel $+ lprior $+ dataselstr $+ subTstr $+ "pm";
open fhpara = ^lpara for read;
para = readr(fhpara,npara);
closeall fhpara;

para_t = para[1:nparat];
para_p = para[nparat+1:npara];

load path = ^priopath parat_names;
load path = ^priopath parap_names;
para_names = parat_names ~ parap_names;

"Model            " $lmodel;
"Data set         " $dataselstr;
"Prior            " $lprior;
"Subsample        " $subTstr;
"MHrun Index      " $mhrun;
"Scaling of Hess  " cc;
"";
"===================================";
"";
"";
"Parameter   | Estimate ";
outmat = para_names'~para;
let mask[1,2] = 0 1;
let fmt[2,3] =
   "-*.*s " 10 4
   "*.*lf " 10 4;
d = printfm(outmat,(0 ~ 1),fmt);

{lnpost0, lnpy0} = fcn(para_t,para_p);

para_tstar = para_t;

"";
"Prior*Likelihood at Mode:" lnpost0;
"Likelihood at Mode      :" lnpy0;
"";
"Press Key to Continue";
k = keyw;
cls;

/******************************************************** 
** Load multiplication matrix for parameters
*/
lmult = postpath $+ "\\" $+ lmodel $+ lprior $+ dataselstr $+ subTstr $+ "mut";
open fhmult = ^lmult for read;
SIGSCALE   = readr( fhmult,nparat );
closeall fhmult;

lhess = postpath $+ "\\" $+ lmodel $+ lprior $+ dataselstr $+ subTstr $+ "hst";
open fhhess = ^lhess for read;
SIGPROPINV  = readr( fhhess,nparat );
closeall fhhess;

SIGPROPdim = sumc(parat_maskinv[.,mspec]);
{u , s, v} = svd1(SIGPROPINV);
SIGPROPlndet = 0;

i = 1;
do until i > nparat;
   if i <= SIGPROPdim;
      s[i,i]    = 1/s[i,i];
      SIGPROPlndet = SIGPROPlndet + ln(s[i,i]);
   endif;
   i = i+1;
endo;

/******************************************************** 
** Initialize Output files
*/

ostat    = opath $+ "\\" $+ lmodel $+ lprior $+ dataselstr $+ subTstr $+ "st"; 
statname = { REJECT Pthmode1 Pthmode2 }; 
create fhstat =^ostat  with ^statname, 3, 8;

otheta = opath $+ "\\" $+ lmodel $+ lprior $+ dataselstr $+ subTstr $+ "pt";
ophi   = opath $+ "\\" $+ lmodel $+ lprior $+ dataselstr $+ subTstr $+ "pp";
oss    = opath $+ "\\" $+ lmodel $+ lprior $+ dataselstr $+ subTstr $+ "ps";

create fhtheta = ^otheta with ^parat_names, nparat, 8;
create fhphi   = ^ophi   with ^parap_names, nparap, 8;
create fhss    = ^oss    with SS, nobs, 2;

/******************************************************** 
**              Metropolis-within-Gibbs Algorithm
*/

/* Initialize SS with values read from file
*/
SS_old    = SS;

/* Initialize phi with posterior mode
*/
phi_old   = para_p;

/* Initialize theta
*/
valid = 0;
do until valid == 1;

   theta_old  = para_t + cc0*(sigscale*rndn(nparat,1));
   theta_old  = theta_old.*parat_maskinv[.,mspec] + parat_fix[.,mspec].*parat_mask[.,mspec];

   {post_theta_old, like_theta_old} = fcn(theta_old,phi_old);
   if post_theta_old > -1E6;
      valid = 1;
   endif; 

endo;   


iblock = 1;
do until iblock > nblocks;

   theta_sim = zeros(nsim,nparat);
   phi_sim   = zeros(nsim,nparap);
   ss_sim    = zeros(nsim,nobs);
   
   like_theta_sim = zeros(nsim,1);
   post_theta_sim = zeros(nsim,1);
   rej            = zeros(nsim,1);
   pthsim         = zeros(nsim,2);

   tstart = date;

   j = 1;

   do until j > nsim;

      /* Step 1: draw from SS | theta, phi
      */
      {ss_new, loglhss}  = drawpostss(theta_old,phi_old,mspec,YY);

      /* Step 2: draw phi | SS
      */
      phi_new = drawpostphi(pmean_p~pstdd_p,ss_new); 

      /* Step 3: draw theta | SS, phi using a Metropolis Step
      */

      theta_new  = theta_old + cc*(sigscale*rndn(nparat,1));
      theta_new  = theta_new.*parat_maskinv[.,mspec] + parat_fix[.,mspec].*parat_mask[.,mspec];

      {post_theta_old, like_theta_old} = fcnss(theta_old,phi_new,SS_new);
      {post_theta_new, like_theta_new} = fcnss(theta_new,phi_new,SS_new);

      propdens_new         = -0.5*SIGPROPDIM*ln(2*pi) - 0.5*SIGPROPlndet - 0.5*SIGPROPDIM*ln(cc^2)
                             -0.5*(theta_new - theta_old)'*SIGPROPINV*(theta_new - theta_old)/cc^2;
                             
      r = minc(1 | exp( post_theta_new - post_theta_old) );
      
      if rndu(1,1) > r;
         /* Reject the proposed jump
         */
         theta_new = theta_old;   
         post_theta_new = post_theta_old;
         rej[j] = 1;
      endif;

      /* Statistics for DD calc according to Chib and Jeliazcov
      */
      {post_thetastar, like_thetastar} = fcnss(para_tstar,phi_new,SS_new);
      alpthsthmode = minc(0 | post_thetastar  - post_theta_new );
      qthsthmode = -0.5*npara*ln(2*pi) - 0.5*SIGPROPlndet - 0.5*npara*ln(cc^2)
                   -0.5*(theta_new - para_tstar)'*SIGPROPINV*(theta_new - para_tstar)/cc^2;

      thtildej   = para_tstar + cc*(sigscale*rndn(nparat,1));
      thtildej   = thtildej.*parat_maskinv[.,mspec] + parat_fix[.,mspec].*parat_mask[.,mspec];

      {posttildej, liketildej} = fcnss(thtildej,phi_new,SS_new);
      alpthstarthj = minc(0 | posttildej - post_thetastar );
     
      pthsim[j,1] = alpthsthmode + qthsthmode;
      pthsim[j,2] = alpthstarthj;

      /* Store results 
      */
      theta_sim[j,.] = theta_new';
      phi_sim[j,.]   = phi_new';
      SS_sim[j,.]    = SS_new[.,2]';
       
      /* Update
      */
      theta_old = theta_new;
      phi_old   = phi_new;
      SS_old    = SS_new;

      /*      
      locate 1,1;
      "Block" iblock "of" nblocks;
      "Simulation step" j "of" nsim;
      "Rejections [%] " 100*meanc(rej[1:j]);
      "THETA "; theta_sim[j,.];
      "PHI   "; phi_sim[j,.]; 
      */

      j = j+1;
   endo;

   tend = date;

   locate 1,1;
   "Block" iblock "of" nblocks;
   "Rejections [percent] " 100*meanc(rej);
   "THETA "; meanc(theta_sim)';
   "PHI   "; meanc(phi_sim)';
   "Elapsed Time"; etstr(ethsec(tstart,tend));

    wr = writer(fhstat,rej~pthsim); 
    wr = writer(fhtheta,theta_sim);
    wr = writer(fhphi,phi_sim);
    wr = writer(fhss,ss_sim);

    iblock = iblock+1;
endo;  

closeall fhtheta, fhphi, fhss;
end;

/****************************************************/
/*                 PROCEDURES                       */
/****************************************************/

proc (2) = fcn(para_t,para_p);
/* Evaluate the likelihood function * prior for para_t, SS integrated out
*/
local lnpY, lnprio, obserror, obsvar, lnpost, retcode, parabd_ind1, parabd_ind2;
 
    parabd_ind1 = para_t .> _bounds[.,1];
    parabd_ind2 = para_t .< _bounds[.,2];
    parabd_ind1 = delif(parabd_ind1,parat_mask[.,mspec]);
    parabd_ind2 = delif(parabd_ind2,parat_mask[.,mspec]);

    if (parabd_ind1 == 1) and (parabd_ind2 == 1);

       {lnpY,retcode,obserror,obsvar} = evallp(para_t,para_p,mspec,YY); 
       lnpy    = real(lnpy);
       lnprio = priodens(para_t, pmean_t, pstdd_t, pshape_t);
       lnpost  = lnpy + real(lnprio);

    else;
      
       lnpost = -1E6;
       lnpy   = -1E20;

    endif;
    
retp(lnpost,lnpy);  
endp;


proc (2) = fcnss(para_t,para_p,SS_draw);
/* Evaluate the likelihood function * prior for para_t conditional on SS
*/
local lnpY, lnprio, obserror, obsvar, lnpost, retcode, parabd_ind1, parabd_ind2;
 
    parabd_ind1 = para_t .> _bounds[.,1];
    parabd_ind2 = para_t .< _bounds[.,2];
    parabd_ind1 = delif(parabd_ind1,parat_mask[.,mspec]);
    parabd_ind2 = delif(parabd_ind2,parat_mask[.,mspec]);

    if (parabd_ind1 == 1) and (parabd_ind2 == 1);

       {lnpY,retcode,obserror,obsvar} = evallpss(para_t,para_p,mspec,YY,SS_draw); 
       lnpy    = real(lnpy);
       lnprio = priodens(para_t, pmean_t, pstdd_t, pshape_t);
       lnpost  = lnpy + real(lnprio);

    else;
      
       lnpost = -1E6;
       lnpy   = -1E20;

    endif;
    
retp(lnpost,lnpy);  
endp;

#include c:\projects\active\learnpol\gauss2\lpmod.src;
#include c:\projects\active\learnpol\gauss2\lplh.src;
#include c:\projects\active\learnpol\gauss2\lpdraw.src;
