/*******************************************************/
/*                                                     */
/* Computing Impulse responses based on                */
/* parameters of DSGE models                           */
/*                                                     */
/*******************************************************/

/* filename:    lpirf.g
** description: The program converts the DSGE parameters
**              into impulse response functions
** created:     
*/

new;
closeall;
library pgraph, user;
cls;

/* Global defaults 
*/
  __output = 0;

#include c:\projects\active\learnpol\gauss2\pathspec.g;
#include c:\projects\active\learnpol\gauss2\lpspec.g;

/*
lmodeluse  = "m3";
mspecuse   = 3;
*/

/* Specify impulse response experiment */
/* Experiment 1
*/
/*
nirf     = 20;
nit      = 500;
RT       = 0.25*ones(1,1);
ST       = 0 ~ 0;
subTstruse = "1";
*/

/* Specify impulse response experiment */
/* Experiment 2
*/
/*
nirf     = 20;
nit      = 500;
RT       = 1*ones(5,1);
ST       = ones(5,1)~zeros(5,1);
subTstruse = "2";
*/

/* Experiment 3
** same as Experiment 1
** except: M3 response is modified - approximation
*/
/*
nirf     = 20;
nit      = 500;
RT       = 0.25*ones(1,1);
ST       = 0 ~ 0;
subTstruse = "3";
*/

/* Experiment 4
** same as Experiment 2
** except: M3 response is modified - approximation
*/
/*
nirf     = 20;
nit      = 500;
RT       = 1*ones(5,1);
ST       = ones(5,1)~zeros(5,1);
subTstruse = "4";
*/

/* Experiment 5
** same as Experiment 1
** except: M3 response is modified: homoskedasticity
*/

nirf     = 20;
nit      = 500;
RT       = 0.25*ones(1,1);
ST       = 0 ~ 0;
subTstruse = "5";


/* Experiment 6
** same as Experiment 2
** except: M3 response is modified: homoskedasticity
*/
/*
nirf     = 20;
nit      = 500;
RT       = 1*ones(5,1);
ST       = ones(5,1)~zeros(5,1);
subTstruse = "6";
*/

/* Open files with Parameter Draws
*/
mhrun  = "2";

lpath  = parapath $+ "\\mhrun" $+ mhrun;
opath  = resupath $+ "\\mhrun" $+ mhrun;

ltheta  = lpath $+ "\\" $+ lmodel $+ lprior $+ dataselstr $+ subTstr $+ "pt";
lphi    = lpath $+ "\\" $+ lmodel $+ lprior $+ dataselstr $+ subTstr $+ "pp";

open fhtheta = ^ltheta for read;
open fhphi   = ^lphi for read;

/* Define some Constants
*/
nblock  = 1000;  /* block size */
nuse    = 200;   /* use every nuse observation */

/* Initialize Output files
*/
oirf = opath $+ "\\" $+ lmodel $+ lprior $+ dataselstr $+ subTstruse $+ "ir";
create fhirf = ^oirf with IRF, 5*nirf, 8;
       
/* Initialization
*/ 
eofloop = 0;
loopct  = 1;

do until eofloop; 

   timestart = date;

   /* Read blocks of size nblock
   */
   thetasim = readr( fhtheta, nblock );
   phisim   = readr( fhphi, nblock);
   
   nread = rows(thetasim);

   indseq = seqa(1,1,nread);
   indseq = indseq % nuse;
   indseq = indseq .NE 0;
 
   thetasim = delif(thetasim,indseq);
   phisim   = delif(phisim,indseq);
   
   nread = rows(thetasim);

   irfsim = zeros(nread,5*nirf);
   irfinvalid = zeros(nread,1);
   
   j = 1;
   do until j > nread;
   
      /* For each element of the block, compute the desired statistic
      */
      
      theta = thetasim[j,.]';
      phi   = phisim[j,.]';
     
      yTirf = lpirf(theta,phi,RT,ST,nirf,nit,mspec);

      if ismiss(YTirf);
         irfinvalid[j,1] = 1;
      endif;   

      irfsim[j,.] = vec(yTirf)';
      j = j+1;
      
   endo;

   /* Write Output
   */
   if sumc(irfinvalid) < rows(irfsim);
      irfsim = delif(irfsim,irfinvalid);
      wr = writer(fhirf,irfsim);
      locate 5,1; meanc(irfsim[.,41:45])';
   endif;

   locate 1,1;
   "Block" loopct;
   "Draws Processed:" nread;   

   loopct = loopct + 1;
   eofloop = eof(fhtheta);

   timeend = date;
   "Elapsed Time"; etstr(ethsec(timestart,timeend));



endo;

closeall fhtheta, fhphi, fhirf;
      
#include c:\projects\active\learnpol\gauss2\lpmod.src;
#include c:\projects\active\learnpol\gauss2\lpirf.src;
