/* filename:    lpmh.g
** description: The program generates draws from the posterior
**              distribution via the random walk Metropolis Hastings Algorithm
**              
** created:     02/21/02
** modified:   
*/

new;
closeall;
library user, pgraph;
cls;

/* Global defaults 
*/
  __output = 1;
  _fcmptol = 1E-10;

#include c:\projects\active\learnpol\gauss2\pathspec.g
#include c:\projects\active\learnpol\gauss2\lpspec.g


/******************************************************** 
**  Configure the Metropolis Algorithm
*/
mhrun   = "3";
cc0     = 1;
cc      = 0.2;
nblocks = 100;
nsim    = 1000;
opath = parapath $+ "\\mhrun" $+ mhrun;

_bounds =
    (1E-5 ~ 10      )|
    (1E-5 ~ 10      )|
    (1E-5 ~ 10      )|
    (1E-5 ~ 10      )|
    (1E-5 ~ 0.99999 )|
    (1E-5 ~ 10      )|
    (1E-5 ~ 10      )|
    (1E-5 ~ 0.99999 )|
    (1E-5 ~ 0.99999 )|
    (1E-5 ~ 0.99999 )|
    (1E-7 ~ 100     )|
    (1E-7 ~ 100     )|
    (1E-7 ~ 100     )|
    (1E-7 ~ 100     )|
    (1E-5 ~ 0.99999 )|
    (1E-5 ~ 0.99999 );

/******************************************************** 
** Import data
** series (nobs,3)
*/
#include c:\projects\active\learnpol\gauss2\loaddata.g

nobs  = rows(series_yT);  /* number of observations */
YY    = series_yT;   

/******************************************************** 
** Import the information for the prior density
*/
priorfile = lmodel $+ lprior $+ dataselstr $+ subTstr $+ "prt.out";
load path = ^priopath prior[nparat,3] = ^priorfile;

pmean_t  = prior[.,1];
pstdd_t  = prior[.,2];
pshape_t = prior[.,3];

pmean_t  = pmean_t.*parat_maskinv[.,mspec] + parat_fix[.,mspec].*parat_mask[.,mspec];
pshape_t = pshape_t.*parat_maskinv[.,mspec];

priorfile = lmodel $+ lprior $+ dataselstr $+ subTstr $+ "prp.out";
load path = ^priopath prior[nparap,3] = ^priorfile;

pmean_p  = prior[.,1];
pstdd_p  = prior[.,2];
pshape_p = prior[.,3];

pmean = pmean_t | pmean_p;
pstdd = pstdd_t | pstdd_p;
pshape = pshape_t | pshape_p;

/******************************************************** 
** Load Posterior Mode Estimates
*/
lpara = postpath $+ "\\" $+ lmodel $+ lprior $+ dataselstr $+ subTstr $+ "pm";
open fhpara = ^lpara for read;
para = readr(fhpara,npara);
closeall fhpara;

load path = ^priopath parat_names;
load path = ^priopath parap_names;
para_names = parat_names ~ parap_names;

"Model            " $lmodel;
"Data set         " $dataselstr;
"Prior            " $lprior;
"Subsample        " $subTstr;
"MHrun Index      " $mhrun;
"Scaling of Hess  " cc;
"";
"===================================";
"";
"";
"Parameter   | Estimate ";
outmat = para_names'~para;
let mask[1,2] = 0 1;
let fmt[2,3] =
   "-*.*s " 10 4
   "*.*lf " 10 4;
d = printfm(outmat,(0 ~ 1),fmt);
{lnpost0, lnpy0} = fcn(para);

"";
"Prior*Likelihood at Mode:" lnpost0;
"Likelihood at Mode      :" lnpy0;
"";
"Press Key to Continue";
k = keyw;
cls;

/******************************************************** 
** Load multiplication matrix for parameters
*/
lmult = postpath $+ "\\" $+ lmodel $+ lprior $+ dataselstr $+ subTstr $+ "mu";
open fhmult = ^lmult for read;
SIGSCALE   = readr( fhmult,npara );
closeall fhmult;

lhess = postpath $+ "\\" $+ lmodel $+ lprior $+ dataselstr $+ subTstr $+ "hs";
open fhhess = ^lhess for read;
SIGPROPINV  = readr( fhhess,npara );
closeall fhhess;

SIGPROPdim = sumc(para_maskinv[.,mspec]);
{u , s, v} = svd1(SIGPROPINV);
SIGPROPlndet = 0;

i = 1;
do until i > npara;
   if i <= SIGPROPdim;
      s[i,i]    = 1/s[i,i];
      SIGPROPlndet = SIGPROPlndet + ln(s[i,i]);
   endif;
   i = i+1;
endo;

/******************************************************** 
** Initialize Output files
*/
ostat = opath $+ "\\" $+ lmodel $+ lprior $+ dataselstr $+ subTstr $+ "st";
statname = { POST LIKE PROP PROPPOST REJECT };
create fhstat=^ostat with ^statname, 5, 8;

otheta = opath $+ "\\" $+ lmodel $+ lprior $+ dataselstr $+ subTstr $+ "pt";
ophi   = opath $+ "\\" $+ lmodel $+ lprior $+ dataselstr $+ subTstr $+ "pp";

create fhtheta = ^otheta with ^parat_names, nparat, 8;
create fhphi   = ^ophi   with ^parap_names, nparap, 8;

/******************************************************** 
**              Metropolis Hastings Algorithm
*/

/* Initialize PARANEW
*/
valid = 0;
do until valid == 1;
   paranew   = para + cc0*(sigscale*rndn(npara,1));
   paranew   = paranew.*para_maskinv[.,mspec] + para_fix[.,mspec].*para_mask[.,mspec];

   {postnew, likenew} = fcn(paranew);
   propdens  = -0.5*SIGPROPDIM*ln(2*pi) - 0.5*SIGPROPlndet - 0.5*SIGPROPDIM*ln(cc0^2)
               -0.5*(paranew - para)'*SIGPROPINV*(paranew - para)/cc0^2;
   if postnew > -10000;
      valid = 1;
   endif;

endo;   


iblock = 1;
do until iblock > nblocks;

   tstart = date;

   parasim = zeros(nsim,npara);
   likesim = zeros(nsim,1);
   postsim = zeros(nsim,1);
   rej     = zeros(nsim,1);
   propsim = zeros(nsim,1);
   proppostsim = zeros(nsim,1);

   if iblock == 1;
      j = 2;
      likesim[1,1] = likenew;
      postsim[1,1] = postnew;
      propsim[1,1] = propdens;
      proppostsim[1,1] = postnew;
      parasim[1,.] = paranew';
      postold      = postnew;
      likeold      = likenew;
      paraold      = paranew;     
   else;
      j=1;
   endif;      

   do until j > nsim;

     paranew          = paraold + cc*(sigscale*rndn(npara,1));
     paranew   = paranew.*para_maskinv[.,mspec] + para_fix[.,mspec].*para_mask[.,mspec];

     {postnew, likenew} = fcn(paranew);
     propdens         = -0.5*SIGPROPDIM*ln(2*pi) - 0.5*SIGPROPlndet - 0.5*SIGPROPDIM*ln(cc^2)
                        -0.5*(paranew - paraold)'*SIGPROPINV*(paranew - paraold)/cc^2;
     propsim[j,1]     = propdens;
     proppostsim[j,1] = postnew;

     r = minc(1 | exp( postnew - postold));
     if rndu(1,1) < r;
       /* Accept proposed jump
       */
       postsim[j,1] = postnew;
       likesim[j,1] = likenew;
       parasim[j,.] = paranew';
       paraold = paranew;
       postold = postnew;
       likeold = likenew;
     else;
       /* Reject proposed jump
       */
       likesim[j,1] = likeold;
       postsim[j,1] = postold;
       parasim[j,.] = paraold';
       rej[j] = 1;
     endif;

     /*
     locate 1,1;
     "Block" iblock "of" nblocks;
     "Simulation step" j "of" nsim;
     "Proposal Dens  " propsim[j,1];
     "Parameters     "; parasim[j,.]; 
     */

     j = j+1;
    endo;

   locate 1,1;
   "Block" iblock "of" nblocks;
   "Rejection: perct " meanc(rej);
   "Likelihood       " meanc(likesim);
   "Posterior        " meanc(postsim);
   "Parameters       "; meanc(parasim)'; 

    wr = writer(fhstat,postsim~likesim~propsim~proppostsim~rej);
    wr = writer(fhtheta,parasim[.,1:nparat]);
    wr = writer(fhphi,parasim[.,nparat+1:npara]);

    tend = date;
    "Elapsed Time"; etstr(ethsec(tstart,tend));

    iblock = iblock+1;
endo;  

closeall fhstat, fhtheta, fhphi;
end;

/****************************************************/
/*                 PROCEDURES                       */
/****************************************************/

proc (2) = fcn(para);
local lnpY, lnprio, obsmean, obsvar, lnpost, retcode, parabd_ind1, parabd_ind2;
 
    parabd_ind1 = para .> _bounds[.,1];
    parabd_ind2 = para .< _bounds[.,2];
    parabd_ind1 = delif(parabd_ind1,para_mask[.,mspec]);
    parabd_ind2 = delif(parabd_ind2,para_mask[.,mspec]);

    if (parabd_ind1 == 1) and (parabd_ind2 == 1);

       {lnpY,retcode,obsmean,obsvar} = evallp(para[1:nparat],para[nparat+1:npara],mspec,YY); 
       lnpy    = real(lnpy);
       lnprio  = priodens( para, pmean, pstdd, pshape);
       lnpost  = lnpy + real(lnprio);

    else;
      
       lnpost = -1E6;
       lnpy   = -1E20;

    endif;
    
retp(lnpost,lnpy);  
endp;

#include c:\projects\active\learnpol\gauss2\lpmod.src;
#include c:\projects\active\learnpol\gauss2\lplh.src;
