/* filename:    lpprior1.g
** description: The program generates draws from the prior of the 
**              learning model
** created:     12/14/01
*/

library pgraph, user;
cls;

block_n    = 20;
block_size = 1000;
mhrun      = "0";   /* NOTE: we use mhrun = 0 to denote prior */
nobs       = 152;


#include c:\projects\active\learnpol\gauss2\pathspec.g;
#include c:\projects\active\learnpol\gauss2\lpspec.g

opath      = parapath $+ "\\mhrun" $+ mhrun;

/******************************************************** 
** Import the information for the prior density
*/
priorfile = lmodel $+ lprior $+ dataselstr $+ subTstr $+ "prt.out";
load path = ^priopath prior[nparat,3] = ^priorfile;

pmean  = prior[.,1];
pstdd  = prior[.,2];
pshape = prior[.,3];
pmean  = pmean.*para_maskinv[.,mspec] + para_fix[.,mspec].*para_mask[.,mspec];
pshape = pshape.*para_maskinv[.,mspec];

priorfile = lmodel $+ lprior $+ dataselstr $+ subTstr $+ "prp.out";
load path = ^priopath prior[nparap,3] = ^priorfile;

pmean_p  = prior[.,1];
pstdd_p  = prior[.,2];
pshape_p = prior[.,3];

/******************************************************** 
**  Initialize Output files
*/

otheta = opath $+ "\\" $+ lmodel $+ lprior $+ dataselstr $+ subTstr $+ "pt0";
ophi   = opath $+ "\\" $+ lmodel $+ lprior $+ dataselstr $+ subTstr $+ "pp0";
oss    = opath $+ "\\" $+ lmodel $+ lprior $+ dataselstr $+ subTstr $+ "ps0";

create fhtheta = ^otheta with THETA, nparat, 8;
create fhphi   = ^ophi   with PHI, nparap, 8;
create fhss    = ^oss    with SS, nobs, 2;

/* Loop to generate parameter draws
*/
block_it = 1;
para_acpt = 0;

do until block_it > block_n;

   /* Generate THETA draws
   */

   parasim_t = zeros(block_size,nparat);

   i = 1; 
   do until i > nparat;
   
      if pshape[i] == 1; /* BETA Prior */
         a = (1-pmean[i])*pmean[i]^2/pstdd[i]^2 - pmean[i];
         b = a*(1/pmean[i] - 1);
         parasim_t[.,i] = rndbeta(block_size,1,a,b);
      
      elseif pshape[i] == 2; /* GAMMA PRIOR */
         b = pstdd[i]^2/pmean[i];
         a = pmean[i]/b;
         parasim_t[.,i] = b*rndgam(block_size,1,a);
      
      elseif pshape[i] == 3; /* GAUSSIAN PRIOR */
         a = pmean[i];
         b = pstdd[i];
         parasim_t[.,i] = a + b*rndn(block_size,1);

      elseif pshape[i] == 4; /* INVGAMMA PRIOR */
         a = pmean[i];
         b = pstdd[i];
         parasim_t[.,i] = sqrt( b*a^2/sumc( (rndn(b,block_size))^2 ) );

      elseif pshape[i] == 5; /* uniform prior */
         a = pmean[i];
         b = pstdd[i];
         parasim_t[.,i] = a + (b-a)*rndu(block_size,1);

      elseif pshape[i] == 0; /* No Prior - Fixed */
         a = pmean[i];
         parasim_t[.,i] = a*ones(block_size,1);

      endif;

      i = i+1;
   endo; 

   /* Generate PHI draws
   */
         
   parasim_p = zeros(block_size,nparap);

   i = 1; 
   do until i > nparap;

      /* Beta Prior */   
      a = (1-pmean_p[i])*pmean_p[i]^2/pstdd_p[i]^2 - pmean_p[i];
      b = a*(1/pmean_p[i] - 1);
      parasim_p[.,i] = rndbeta(block_size,1,a,b);
      
      i = i+1;
   endo; 

   /* Generate SS draws
   */
   
   parasim_ss = zeros(block_size,nobs);
   
   i = 1;
   do until i > block_size;
 
      ssdraw = drawpriorss(parasim_p[i,.]',nobs);
      parasim_ss[i,.] = ssdraw[.,2]';

      i = i+1;

   endo;

   locate 1,1;
   "Processing Block" block_it;
   
   wr = writer(fhtheta,parasim_t);
   wr = writer(fhphi,parasim_p);
   wr = writer(fhss,parasim_ss);

   block_it = block_it + 1;

endo;

closeall fhtheta, fhphi, fhss;
end;

#include c:\projects\active\learnpol\gauss2\lpmod.src;
#include c:\projects\active\learnpol\gauss2\lpdraw.src;
