dlibrary -a PszTgSen,PsdTgSen;

/* Model Specification
**
** 1 = full information about policy regime, rational expectations about future policy
** 2 = full information about policy regime, myopic expectations about future policy
** 3 = limited information about policy regime, (limited) rational exp about future policy
**
*/
mspec = 3;
lmodel = "m" $+ ftos(mspec,"%*.*lf",1,0) ;

mprior = 1;
lprior = ftos(mprior,"%*.*lf",1,0);

/*
** para_mask is npara * (number of model specifications)
**   to fix a parameter under specification mspec, set the corresponding
**   element of para_mask to 1
** para_fix is npara * (number of model specifications)
**   and contains the values for the fixed parameters (all other entries
**   are irrelevant
*/
nparat  = 14;
nparap  = 2;
npara = nparat + nparap;

parat_mask    = zeros(nparat,4); /* parat_mask[nparat,3]=1; */
parat_maskinv = ones(nparat,4) - parat_mask;
parat_fix     = zeros(nparat,4);

parap_mask    = zeros(nparap,4);
parap_maskinv = ones(nparap,4) - parap_mask;
parap_fix     = zeros(nparap,4);

para_mask    = parat_mask | parap_mask;
para_maskinv = parat_maskinv | parap_maskinv;
para_fix     = parat_fix | parap_fix;


/* Selected data set
** 0 = actual, full sample
** i = simulated data set i
*/
datasel    = 0;
dataselstr = ftos(datasel,"%*.*lf",1,0);

/* FYI: Subperiods in Clarida Gali and Gertler
** Pre-Volcker:       1960:1 to 1979:2, obs  1 -  78
** Volcker-Greenspan: 1979:3 to 1997:4, obs 79 - 152
** post-82:           1982:4 to 1997:4, obs 92 - 152
*/

/* Use subT to select subsample
*/

if datasel < 1; /* actual data */

   subT = 1;   
   subTstr = ftos(subT,"%*.*lf",1,0);
   
else; /* simulated data */

   Tsimtotal = 100000;
   Tsimburn  = 1000;
   Tsim = 200 | 500;
   subT = 1;   
   subTstr = ftos(subT,"%*.*lf",1,0);
     
endif;

     
/* Number of draws to compute mean and variance of regime prob
*/
epsrtdraw_n = 500; 
