/* filename:    mhdd.g
** description: Computes numerical approximations to the data density
**              based on MCMC output
*/

new;
closeall;
library user, pgraph;
cls;
outwidth 128;

/******************************************************************
**         Load Parameter Draws from MH Output
*/
#include c:\projects\active\learnpol\gauss2\pathspec.g;
#include c:\projects\active\learnpol\gauss2\lpspec.g;

mhrun = "2";

lpath  = parapath $+ "\\mhrun" $+ mhrun;
lstat  = lpath $+ "\\" $+ lmodel $+ lprior $+ dataselstr $+ subTstr $+ "st";
ltheta = lpath $+ "\\" $+ lmodel $+ lprior $+ dataselstr $+ subTstr $+ "pt";
lphi   = lpath $+ "\\" $+ lmodel $+ lprior $+ dataselstr $+ subTstr $+ "pp";

open fhtheta = ^ltheta for read;
open fhphi   = ^lphi for read;
open fhstat = ^lstat for read;

nburn   = 20001;
nblock  = 10000;
densfac = -700;
hmax    = 20;

npara   = sumc(para_maskinv[.,mspec]);

pararow = seekr(fhtheta,nburn);
pararow = seekr(fhphi,nburn);
statrow = seekr(fhstat,nburn);

/* 
** Method:  modified harmonic mean
*/

p = seqa(0.1,0.1,9); 
pcrit = cdfchii(p,ones(rows(p),1)*npara);

/* Part 1: Compute posterior moments for parameters
*/

eofloop = 0;
ndraws  = 0;
sumpara  = zeros(npara,1);
sumpara2 = zeros(npara,npara);

do until eofloop; 

   parasim = readr( fhtheta, nblock ) ~ readr( fhphi, nblock) ;
   parasim = delif(parasim',para_mask[.,mspec])';

   sumpara  = sumpara + sumc(parasim);
   sumpara2 = sumpara2 + parasim'*parasim;
   ndraws   = ndraws + rows(parasim);

   eofloop = eof(fhtheta);

endo;

parahat       = sumpara / ndraws;
parasig       = sumpara2 / ndraws - parahat*parahat';
parasiginv   = inv(parasig);
parasiglndet = ln(det(parasig));


/* Part 2:  Compute Density estimate
*/

pararow = seekr(fhtheta,nburn);
pararow = seekr(fhphi,nburn);
statrow = seekr(fhstat,nburn);

eofloop = 0;
ndraws  = 0;
suminvlike = zeros(rows(p),1);
laginvlike = zeros(hmax,rows(p));
gaminvlike = zeros(hmax,rows(p));

do until eofloop;

   statsim  = readr( fhstat, nblock);
   parasim = readr( fhtheta, nblock) ~ readr( fhphi, nblock);
   parasim = delif(parasim',para_mask[.,mspec])';

   i = 1;
   do until i > rows(parasim);
    
      lnfpara = -ln(p) - 0.5*npara*ln(2*pi) - 0.5*parasiglndet
                 -0.5*(parasim[i,.] - parahat')*parasiginv*(parasim[i,.]' - parahat);
      indpara = ((parasim[i,.] - parahat')*parasiginv*(parasim[i,.]' - parahat) .< pcrit );
      invlike = exp(lnfpara - statsim[i,1] + densfac).*indpara;

      laginvlike = invlike' | laginvlike[1:hmax-1,.];
      gaminvlike = gaminvlike + laginvlike.*invlike';

      suminvlike = suminvlike + invlike;
      i = i+1;
   endo;

   ndraws = ndraws + rows(parasim);
   locate 1,1;
   ndraws;
   eofloop = eof(fhtheta);

endo;
closeall fhtheta, fhphi, fhstat;

meaninvlike = suminvlike/ndraws;
gaminvlike = gaminvlike/ndraws - (meaninvlike^2)';

/* Compute Standard Errors
*/
suminvlikeerror = gaminvlike[1,.];
j = 2;
do until j > hmax;
   suminvlikeerror = suminvlikeerror + 2*gaminvlike[j,.]*(1- (j-1)/hmax);
   j = j+1;
endo;
suminvlikeerror = 100*sqrt(suminvlikeerror/ndraws)./meaninvlike' ;

pym3        = densfac-ln(meaninvlike);

"Model:  " $lmodel;
"Prior:  " $lprior;
"MhRun:  " $mhrun;
"Dataset:" $dataselstr;
"======================";
"";

"Marginal Data Density" pym3;
"Average Across p     " meanc(pym3);
"Simulation Error [Percent]" suminvlikeerror';

end;


