/* filename:    ph_conv.g
** description: Informal assessment of convergence
**              plot recursive and moving averages
*/

new;
closeall;
library user, pgraph;
cls;
outwidth 128;

mhrun  = "1";

nburn       = 11;        /* Number of initial draws to be discarded */
nblock      = 200;
nma         = 10;

/******************************************************************
**         Load Parameter Draws from MH Output
*/
#include c:\projects\active\learnpol\gauss2\pathspec.g;
#include c:\projects\active\learnpol\gauss2\lpspec.g;

lpath  = parapath $+ "\\mhrun" $+ mhrun;
ldraw  = lpath $+ "\\" $+ lmodel $+ lprior $+ dataselstr $+ subTstr $+ "pp";
open fhdraw = ^ldraw for read;

load path = ^priopath parap_names;

drawrow = readr( fhdraw, nburn); 
drawdim = cols(drawrow);
drawrow = seekr( fhdraw, nburn);

graphset;
fonts("microb");
_ptitlht = 0.4;
_paxht = 0.4;
_pnumht = 0.4;
_pltype = {6 1 2}; 
_plegstr = "Cumulative Average\000Moving Average"; 
_plegctl = {2 4 4 4};
_plwidth = 3.5;

begwind;
window(1,2,0);

j = 1;
do until j > drawdim;

   /* Read only the j'th column
   */
   drawrow   = seekr( fhdraw, nburn); 
   drawblock = readr( fhdraw, 1);

   drawcol = drawblock[1,j];
   eofloop = 0;
   ndraws  = 1;
   
   do until eofloop; 

      drawblock = readr( fhdraw, nblock );
      drawcol   = drawcol | drawblock[.,j];
      ndraws    = ndraws + rows(drawblock);
      eofloop   = eof(fhdraw);
               
   endo;
   
   draw_cummean = cumsumc(drawcol)./seqa(1,1,ndraws);
   draw_ma      = movave(drawcol,nma);
   draw_ma      = draw_cummean[1:nma-1,. ] | draw_ma;

   title(parap_names[j]);
   xy(seqa(1,1,ndraws),draw_cummean~draw_ma);
   nextwind;

   j = j+1;
   clear drawcol;
   
endo;

endwind;

closeall fhdraw;

end;

/**************************************************************/
/*                         Procedures                         */
/**************************************************************/

proc(1) = movave(draws,nma);
local drawdim, ndraws, mavec, i, j;    

drawdim   = cols(draws);
ndraws    = rows(draws);

mavec = zeros(1+ndraws-nma,drawdim);

i = 1;
do until i > drawdim;

   j = 1;
   do until j > (1+ndraws-nma);
      mavec[j,i] = meanc(draws[j:j+nma-1,i]);
      j = j+1;
   endo;

   i = i+1;
endo;

retp(mavec);
endp;
